/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf;

import java.util.Map;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.util.Fractal;

/**
 * The default BindingFactory implementation. This implementation can export and
 * bind components (creating skeleton and stub components binding them to the
 * server and the client, respectively).
 * 
 * The binding factory use a map of hints to load the plugin to use during the
 * generation of the binding components.
 */
public class BindingFactoryImpl implements BindingFactory, BindingController {

	/**
	 * The logger.
	 */
	Logger log = Logger.getLogger(BindingFactoryImpl.class.getCanonicalName());

	/**
	 * The pluginResolver returns the plug-in for the specified
	 * {@link ExportMode}.
	 */
	PluginResolver pluginResolver;

	/**
	 * Export an interface using the specified hints. The export process is
	 * peformed by executing the following steps:
	 * <ol>
	 * <li>input parameters are verified;
	 * <li>a lookup of the functional interface whose name is 'itfName' is done
	 * on the input serverComponent passed as parameter;
	 * <li>using the hints, the appropriate plug-in is loaded
	 * <li>the loaded plugin is used to generate a skeleton component
	 * <li>the skeleton is added to the enclosing componento of the
	 * serverComponent passed as parameter
	 * <li>the client inteface named 'delegate' owned by the skeleton is bound
	 * to the functional interface
	 * </ol>
	 * 
	 * @param serverComponent
	 *            the component to export
	 * @param itfName
	 *            the name of the interface to export
	 * @param hints
	 *            hits used to export the interface. See {@link ExportMode} for
	 *            further references.
	 * @throws BindingFactoryException
	 */
	public void export(Component serverComponent, String itfName,
			Map<String, String> hints) throws BindingFactoryException {

		// 0) verify parameters
		if (serverComponent == null) {
			IllegalArgumentException iea = new IllegalArgumentException(
					"The reference to the owner of the interface to export was null");
			throw new BindingFactoryException(iea);

		} else if (itfName == null || itfName.equalsIgnoreCase("")) {
			IllegalArgumentException iea = new IllegalArgumentException(
					"The name of the server interface to export can't be null or empty (was:"
							+ itfName + ")");
			throw new BindingFactoryException(iea);

		}

		// 1) retrieve the Fractal interface for the specified 'itfName'
		Object interfaceToExport = null;

		try {
			interfaceToExport = serverComponent.getFcInterface(itfName);
		} catch (NoSuchInterfaceException e) {
			throw new BindingFactoryException("Cannot find interface '"
					+ itfName
					+ "' on the component owning the interface to export", e);
		}
		// 2) load the the plug-in to handle the given export mode
		BindingFactoryPlugin plugin = loadPlugin(hints);

		if (plugin == null) {
			throw new BindingFactoryException(
					"Couldn't load a plugin using hints: " + hints.toString());
		}

		ExportHints exportHints = plugin.getExportHints();

		// 3) use the plugin to generate the skeleton
		Component skel = null;
		try {
			skel = plugin.createSkel(itfName, interfaceToExport,
					serverComponent, exportHints);
		} catch (SkeletonGenerationException e) {
			throw new BindingFactoryException(
					"Some error occured while creating the skeleton using the "
							+ plugin + " plugin ", e);
		}

		try

		{
			// 4) get the the composite component containing the server
			// component
			// being exported
			Component parentComposite = getParentComponent(serverComponent);

			if (parentComposite == null) {
				throw new BindingFactoryException(
						"Cannot export a an interface declared by a component not being contained into a composite component");
			}

			// 5) add the skeleton into the same composite of the server
			// component
			Fractal.getContentController(parentComposite).addFcSubComponent(
					skel);

			// 6) bind the skeleton 'delegate' client interface to the
			// 'component'
			// interface of the serverComponent
			Fractal.getBindingController(skel).bindFc("delegate",
					serverComponent);
			log.finest("Delegate reference used by skeleton: "
					+ Fractal.getBindingController(skel).lookupFc("delegate"));

			plugin.finalizeSkeleton(skel, hints);

			try {
				String compName = Fractal.getNameController(serverComponent)
						.getFcName();
				log.info("BindingFactory successfully exported interface "
						+ itfName + " owned by component [" + compName
						+ "] with the following parameters: " + hints);
			} catch (NoSuchInterfaceException e) {
				log.info("Component [unnamed] exports its interface " + itfName
						+ " with the following parameters: " + hints);

			}

			// start the skeleton
			Fractal.getLifeCycleController(skel).startFc();
		} catch (NoSuchInterfaceException e) {
			throw new BindingFactoryException(
					"Some error occurred while accessing the membrane of the component",
					e);
		} catch (IllegalBindingException e) {
			throw new BindingFactoryException(
					"Some error occurred while binding the skeleton to the component",
					e);
		} catch (IllegalLifeCycleException e) {
			throw new BindingFactoryException(
					"Some error occurred while binding the skeleton to the component interface",
					e);
		} catch (IllegalContentException e) {
			throw new BindingFactoryException(
					"Some error occurred while adding the skeleton into the composite containing the component owning the interface being exported",
					e);
		}

	}

	/**
	 * @param serverComponent
	 * @return
	 * @throws NoSuchInterfaceException
	 */
	private Component getParentComponent(Component serverComponent)
			throws NoSuchInterfaceException {
		Component parentComposite = Fractal.getSuperController(serverComponent)
				.getFcSuperComponents()[0];
		return parentComposite;
	}

	/**
	 * Load a {@link BindingFactoryPlugin}. The map used to load the plugin must
	 * contain a value for key {@link ExportMode#EXPORT_MODE}. If such key is
	 * not found, the plugin to establish local bindings is used instead.
	 * 
	 * @param hints
	 *            contains hints to load the plugin
	 * @return the plugin
	 * @throws BindingFactoryException
	 *             if the pluginResolver client interface is not bound, or if
	 *             some problem occurred while loading the specified plugin
	 *             using a specific export mode
	 */
	protected BindingFactoryPlugin<?, ?> loadPlugin(Map<String, String> hints)
			throws BindingFactoryException {

		String exportMode = hints.get(ExportMode.EXPORT_MODE);

		// load appropriate plugin for the specified export mode
		BindingFactoryPlugin<?, ?> plugin;
		try {
			if (pluginResolver == null) {
				throw new BindingFactoryException(
						"The PluginResolver component couldn't be loaded");

			}
			plugin = pluginResolver.resolvePlugin(exportMode);
		} catch (PluginNotFoundException e) {
			throw new BindingFactoryException(
					"Cannot resolve any plugin for the export id: "
							+ exportMode, e);
		}
		plugin.setInitialHints(hints);
		return plugin;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactory#bind(org.objectweb.fractal.api.Component,
	 *      java.lang.String, java.util.Map)
	 */
	public void bind(Component component, String clientItf,
			Map<String, String> hints) throws BindingFactoryException {

		BindingFactoryPlugin<?, ?> plugin = loadPlugin(hints);
		// create an appropriate stub
		final Component stub = plugin.createStub(component, clientItf, hints);

		finalizeBind(component, clientItf, stub, plugin, hints);

	}

	/**
	 * Add the stub component into the the same composite component of the
	 * client component, and bind the client component interface to the
	 * appropriate interface of the stub.
	 * 
	 * @param clientComponent
	 *            the component owning the client interface
	 * @param clientItf
	 *            the name of the client interface the stub is for
	 * @param stub
	 *            a stub for the client interface
	 * @param plugin
	 *            the plug-in to use to finalize the binding
	 * @param finalizationHints
	 *            the map of hints to be used during the finalization phase
	 * @throws BindingFactoryException
	 *             if some exception occurs
	 */
	private void finalizeBind(Component clientComponent, String clientItf,
			final Component stub, BindingFactoryPlugin<?, ?> plugin,
			Map<String, String> finalizationHints)
			throws BindingFactoryException {
		try {

			// get the parent of the client component
			Component parentOfClient = getParentComponent(clientComponent);

			// insert the stub into same composite of the client
			Fractal.getContentController(parentOfClient)
					.addFcSubComponent(stub);

			// ask the plugin to finalize the stub, if required
			plugin.finalizeStub(stub, finalizationHints);

			// bind the client interface of the client to the server interface
			// of the stub
			Fractal.getBindingController(clientComponent).bindFc(clientItf,
					stub.getFcInterface(clientItf));
			log.fine("Client interface: " + clientItf + " owned by: "
					+ clientComponent
					+ " is now bound to the generated stub component:" + stub
					+ " throught its server interface:" + clientItf);

		} catch (Exception e) {
			throw new BindingFactoryException(
					"Some error occurred while finalizing the binding", e);
		}
	}

	/**
	 * Unexport an interface, removing the skeleton from the membrane. If there
	 * are established bindings with the interface to be unexorted, the
	 * operation is interrupted. More complicate policies can be established in
	 * future.
	 * 
	 * @param c
	 *            the component owning the interface to be unexported
	 * @param itfName
	 *            the name of the interface to unexport
	 * @param hints
	 *            hints to be used to unexport the interface
	 * @throws BindingFactoryException
	 *             if some error occurs while unexporting the interface
	 */
	public void unexport(Component c, String itfName, Map<String, String> hints)
			throws BindingFactoryException {
		throw new UnsupportedOperationException(

		);
	}

	/**
	 * Unbind a client interface.
	 * 
	 * @param component
	 *            the component of the interface to unbind
	 * @param clientItf
	 *            the name of the client inteface to unbind.
	 */
	public void unbind(Component component, String clientItf)
			throws BindingFactoryException {
		throw new UnsupportedOperationException(

		);
	}

	// ///////////////////
	// BINDING-CONTROLLER
	// ///////////////////
	/**
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
	 *      java.lang.Object)
	 */
	public void bindFc(String clientItf, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {
		if (clientItf.equalsIgnoreCase(PluginResolver.INTERFACE_NAME)) {
			this.pluginResolver = (PluginResolver) serverItf;
			return;
		}
		throw new NoSuchInterfaceException("Cannot find client interface "
				+ clientItf);
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc() {
		return new String[] { PluginResolver.INTERFACE_NAME };
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItf) throws NoSuchInterfaceException {
		if (clientItf.equalsIgnoreCase(PluginResolver.INTERFACE_NAME)) {
			return this.pluginResolver;
		}
		throw new NoSuchInterfaceException("Cannot find client interface "
				+ clientItf);
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public void unbindFc(String clientItf) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {
		if (clientItf.equalsIgnoreCase(PluginResolver.INTERFACE_NAME)) {
			this.pluginResolver = null;
			return;
		}
		throw new NoSuchInterfaceException("Cannot find client interface "
				+ clientItf);

	}

}
