/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf;

import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.bf.proxies.StubComponentFactory;

/**
 * An interface to model a binding factory plugin.
 * 
 * @author Valerio Schiavoni <valerio.schiavoni@gmail.com>
 */
public interface BindingFactoryPlugin<E extends ExportHints, B extends BindHints> {

	public static final String IDENTIFIER = "identifier";

	/**
	 * This method creates plugin specific skeletons. The advantage of returning
	 * a component is that this can be reconfigured at runtime.
	 * 
	 * @param interfaceName
	 *            the name of the interface to export
	 * @param interfaceToExport
	 *            a reference to the the interface to export
	 * @param owner
	 *            the component owning the interface to export
	 * @param exportHints
	 *            hints used to create the skeleton
	 * @return the skeleton component
	 * @throws SkeletonGenerationException
	 *             if some error occurs during the generation of the skeleton
	 */
	Component createSkel(String interfaceName, Object interfaceToExport,
			Component owner, E exportHints) throws SkeletonGenerationException;

	/**
	 * This method creates plugin specific stubs. The advantage of returning a
	 * component is that this can be reconfigured at runtime.
	 * 
	 * @param client
	 *            the component owning the interface to export
	 * @param itfname
	 *            the name of the client interface to bound to the stub
	 * @param hints
	 *            hints to create the stub
	 * @throws BindingFactoryException
	 */
	Component createStub(Component client, String itfname,
			Map<String, String> hints) throws BindingFactoryException;

	/**
	 * Finalize the stub component, setting bindings or performing operations
	 * that couldn't be done before.
	 * 
	 * @param stub
	 *            the stub component to finalize
	 * @param finalizationHints
	 *            hints to be used during the finalization process
	 * @throws BindingFactoryException
	 */
	void finalizeStub(Component stub, Map<String, String> finalizationHints)
			throws BindingFactoryException;

	/**
	 * Finalize the skel component, setting bindings or performing operations
	 * that couldn't be done before.
	 * 
	 * @param skel
	 *            the skeleton component to finalize
	 * @param finalizationHints
	 *            hints to be used during the finalization process
	 * @throws BindingFactoryException
	 */
	void finalizeSkeleton(Component skel, Map<String, String> finalizationHints)
			throws BindingFactoryException;

	/**
	 * The stub component factory required by this plug in.
	 * 
	 * @return stub component factory required by this plug in.
	 */
	StubComponentFactory getStubComponentFactory();

	/**
	 * @return the {@link ExportHints} specific for this connector.
	 */
	ExportHints getExportHints();

	/**
	 * 
	 * @return the {@link BindHints} specific for this connector.
	 */
	BindHints getBindHints();

	String getPluginIdentifier();

	public Map<String, String> getInitialHints();

	public void setInitialHints(Map<String, String> initialHints);
}
