/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.proxies;

import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.bf.BindHints;
import org.objectweb.fractal.julia.type.BasicInterfaceType;
import org.objectweb.fractal.util.Fractal;

/**
 * Abstract implementation for a {@link StubComponentFactory
 * StubComponentFactory}.
 * 
 */
public abstract class AbstractStubComponentFactory implements
		StubComponentFactory {
	/**
	 * A logger
	 */
	Logger log = Logger.getLogger(AbstractStubComponentFactory.class.getName());
	/**
	 * The type factory used to instantiate component types
	 */
	protected TypeFactory tf;
	/**
	 * The generic factory used to instantiate components
	 */
	protected GenericFactory gf;

	/**
	 * Abstract implementation for proxy factories.
	 */
	public AbstractStubComponentFactory() {
		super();
		try {
			this.tf = Fractal.getTypeFactory(Fractal.getBootstrapComponent());
			this.gf = Fractal
					.getGenericFactory(Fractal.getBootstrapComponent());
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}

	}

	public Component createStubComponent(InterfaceType iType,
			BindHints bindHints) throws InstantiationException {

		log
				.fine("Creating stub component from client interface's interface type: "
						+ iType + " and bindHints: " + bindHints);

		Class<?> clazz = null;
		try {
			clazz = Thread.currentThread().getContextClassLoader().loadClass(
					iType.getFcItfSignature());
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			throw new InstantiationException(
					"Cannot find class for interface type: "
							+ iType.getFcItfSignature());
		}

		// Class proxyClass = null;
		Object stubComponentContentInstance = null;
		try {
			// generate the
			stubComponentContentInstance = getStubContentGenerator()
					.getStubComponentContentInstance(clazz, bindHints);
		} catch (GeneratorException e) {

			e.printStackTrace();
			throw new InstantiationException(
					"Some error occured while generating proxyClass: "
							+ e.getCause());
		}

		InterfaceType stubInterfaceType = symmetricInterfaceType(iType);
		log.finer("Interface type for stub component created: "
				+ stubInterfaceType);
		ComponentType stubComponentType = createProxyType(stubInterfaceType);
		log.finer("Component type for stub component created: "
				+ stubComponentType);
		Object[] ctrlDesc = new Object[] {
				stubComponentContentInstance.getClass().getClassLoader(),
				proxyMembraneDescriptor() };
		log.finest("Controller descriptor for stub component: " + ctrlDesc);
		Component stubComponent = gf.newFcInstance(stubComponentType, ctrlDesc,
				stubComponentContentInstance);
		log.finer("Stub component created: " + stubComponent);

		try {
			final String stubComponentFcName = clazz.getCanonicalName()
					+ "$$BindingFactoryGeneratedStub";
			Fractal.getNameController(stubComponent).setFcName(
					stubComponentFcName);
			log.finer("Stub component name set to: " + stubComponentFcName);
		} catch (NoSuchInterfaceException e) {
			// do nothing if no name controller is present
			log
					.warning("Cannot set the name of the stub component - (cannot find NameController interface)");
		}

		return stubComponent;
	}

	/**
	 * Override this method if your stub requires a component membrane other
	 * than 'primitive'.
	 * 
	 * @return the string descriptor for the proxy component.
	 */
	protected String proxyMembraneDescriptor() {
		return "primitive";
	}

	/**
	 * @param proxyInterfaceType
	 *            the interface type for the concrete proxy
	 * @return the component type of the proxy component
	 * @throws InstantiationException
	 */
	protected ComponentType createProxyType(InterfaceType proxyInterfaceType)
			throws InstantiationException {
		ComponentType stubComponentType = tf
				.createFcType(new InterfaceType[] { proxyInterfaceType });
		return stubComponentType;
	}

	/**
	 * Template method, to be overridden by concrete subclasses of
	 * {@link AbstractStubComponentFactory} factory class.
	 * 
	 * 
	 * @return the stubContentGenerator to be used during the creation of the
	 *         Stub component
	 * 
	 */
	protected abstract <B extends BindHints> StubContentGenerator<B> getStubContentGenerator();

	/**
	 * Return a copy of the given interface type where the direction of the
	 * interface has been inverted: if the direction of the specified interface
	 * type is client (resp. server), the direction of the returned interface
	 * type is server (resp. client).
	 */
	private static InterfaceType symmetricInterfaceType(InterfaceType it) {

		return new BasicInterfaceType(it.getFcItfName(),
				it.getFcItfSignature(), !it.isFcClientItf(), // client <->
				// server
				it.isFcOptionalItf(), it.isFcCollectionItf());
	}

}